"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pg_1 = require("pg");
exports.createDb = (config) => __awaiter(this, void 0, void 0, function* () {
    const pool = new pg_1.Pool(config);
    let lastActionId = 0;
    yield pool.connect();
    yield pool.query(`
    CREATE TABLE IF NOT EXISTS actions (
      action_id serial    NOT NULL,
      sent_at   timestamp NOT NULL DEFAULT NOW(),
      action    text      NOT NULL
    )
  `);
    yield pool.query(`
    CREATE TABLE IF NOT EXISTS rooms (
      room_id    text      NOT NULL UNIQUE,
      state      text      NOT NULL,
      updated_at timestamp NOT NULL DEFAULT NOW()
    )
  `);
    const db = {
        pushAction: (action) => {
            const strAction = JSON.stringify(action);
            pool.query(`
        INSERT INTO actions (action)
        VALUES ($1::text)
      `, [strAction])
                .catch(console.log);
        },
        fetchState: (roomId) => __awaiter(this, void 0, void 0, function* () {
            const result = yield pool.query(`
        SELECT *
        FROM rooms
        WHERE room_id = $1::text
      `, [roomId]);
            if (result.rows.length === 0)
                return { confirmed: undefined, members: [], lastUpdatedAt: 0 };
            const strState = result.rows[0].state;
            return JSON.parse(strState);
        }),
        stashState: (roomId, state) => {
            const strState = JSON.stringify(state);
            pool.query(`
        INSERT INTO rooms (room_id, state)
        VALUES($1::text, $2::text)
        ON CONFLICT (room_id) DO UPDATE
        SET state = $2::text,
            updated_at = NOW()
        WHERE rooms.room_id = $1::text
      `, [roomId, strState])
                .catch(console.log);
        }
    };
    const loadNewActions = () => __awaiter(this, void 0, void 0, function* () {
        const result = yield pool.query(`
      SELECT *
      FROM actions
      WHERE action_id > $1::int
    `, [lastActionId]);
        result.rows.forEach(row => {
            const action = JSON.parse(row.action);
            action.$timeMs = row.sent_at;
            lastActionId = row.action_id;
            if (db.onaction)
                db.onaction(action);
        });
    });
    const removeOldActions = () => __awaiter(this, void 0, void 0, function* () {
        yield pool.query(`
      DELETE FROM actions
      WHERE sent_at < NOW() - interval '5 minute'
    `);
    });
    setInterval(loadNewActions, 1000);
    setInterval(removeOldActions, 60000);
    return db;
});
